/**
 * @Name WonderfulChat Server
 * @Author Adam Boutcher
 * @Date 20-03-2011
 * @Version 0.9 RC
 */

// Lets do some importing!
import java.io.*;
import java.net.*;
import java.util.*;
import java.sql.Timestamp;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;

public class ChatServer
{
	// Variable time!
		//Versioning
		private static final double Version = 0.9;			// Version Number
		private static final String VType = "   RC";		// Version Type; "Alpha", " Beta", "   RC", "Final"
		// Socket related
		private static final int SSLport = 555;				// SSL Port
		private static int PortCounter = 0;					// Port Counter
		private static SSLServerSocket SSLSock;				// SSL Socket
		// Hashtable
		private Hashtable outputStreams = new Hashtable();	// Socket Mapping
		
	// Standard Msg/Error Output
		public static void ServerMsg(String $type, String $message)
		{
			//We want time stamps on this incase we need to log it leter on.
			java.util.Date date= new java.util.Date();
			System.out.println("["+new Timestamp(date.getTime())+"] "+$type+": "+$message);
		}
	
	// Some Sexy Header
		public static void Header()
		{
			System.out.print("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");	// Horrible hacky way to clear the screen that is OS independent
			System.out.println("##############################################");
			System.out.println("#                                            #");
			System.out.println("#                                            #");
			System.out.println("#             WonderfulChat Server           #");
			System.out.println("#               Java-Based Chat              #");
			System.out.println("#                                            #");
			System.out.println("#                                            #");
			System.out.println("# v"+Version+"                                 "+VType+" #");
			System.out.println("##############################################");
			System.out.println("                     Created by: Adam Boutcher");
			System.out.println("                  Website: www.aboutcher.co.uk\n");
			ServerMsg("Msg", "Server Starting...\n");
		}
		
	// Secure port bindings
		public static void BindSSL()
		{
			try
			{
				SSLServerSocketFactory SSLSockFac = (SSLServerSocketFactory) SSLServerSocketFactory.getDefault();
				SSLSock = (SSLServerSocket) SSLSockFac.createServerSocket(SSLport);
				ServerMsg("Msg", "Secure Connections bound to port "+SSLport+".");
			}
			catch (IOException e)
			{
				ServerMsg("\nError", "0001 - Unable to bind to port ("+SSLport+").");
				ServerMsg("Error", "Secure Connections Unavailable.\n");
				PortCounter++;
			}
		}
		
	// Check Available Ports
		public static void Startup()
		{
			if (PortCounter == 1)
			{
				ServerMsg("Error", "0003 - Exiting, cannot attach to port.\n");
				System.exit(1);
			}
			else
			{
				ServerMsg("Msg", "Server Succefully Started!\n");
			}
		}

	// Listening for Secure Connections
		public ChatServer() throws IOException
		{
			ServerMsg("Msg", "Listening for Secure Conenctions...");
			do
			{			
				SSLSocket client = (SSLSocket) SSLSock.accept();
				ServerMsg("Msg", "Client Securely Connected.");
				DataOutputStream DataOut = new DataOutputStream(client.getOutputStream());
				outputStreams.put(client,DataOut);
				new ServerThread(this,client);
			}while(true);
		}
	// List the streams ready to send stuff to them.
		Enumeration getOutputStreams()
		{
			return outputStreams.elements();
		}
	// Send the message to all the clients connected
		void sendToAll(String message)
		{
			//Sync the streams/connections just in case a client has disconnected and this will help stop things breaking
			synchronized(outputStreams)
			{
				//Go through the array of clients and send the message.
				for (Enumeration e = getOutputStreams(); e.hasMoreElements();)
				{
					DataOutputStream DataOut = (DataOutputStream)e.nextElement();
					try
					{
						DataOut.writeUTF(message);
					}
					catch(IOException ie)
					{
						// If its breaks totally
						ServerMsg("\nError","0005 - Cannot send message to client.\n");
					}
				}
			}
		}
	// Remove a client
		void removeConnection(Socket client)
		{
			// Sync the streams/client list so it doesn't go bang
		    synchronized(outputStreams)
		    {
		    	ServerMsg("Msg","Client Disconnected.");
		    	outputStreams.remove(client);
		    	try
		    	{
		    		client.close();
		    	}
		    	catch(IOException ie)
		    	{
		    		// If there is an error removing/disconnecting a client
		    		ServerMsg("\nError","0200 - Client disconnecting error.\n");
		    	}
		   }
		}
	// The main fun	
		public static void main(String[] args) throws IOException
		{
			// Display that sexy header we made earlier
			Header();
			// Port stuff
			BindSSL();
			// Check ports available
			Startup();
			new ChatServer();
		}
}
class ServerThread extends Thread
{
	// Variable time!
		private ChatServer server;
		private Socket socket;  
	// Standard Msg/Error Output
		public static void ServerMsg(String $type, String $message)
		{
			java.util.Date date= new java.util.Date();
			System.out.println("["+new Timestamp(date.getTime())+"] "+$type+": "+$message);
		}
	 // Contructing the server tread for each client
		public ServerThread(ChatServer chatServer, Socket socket)
		{
			this.server = chatServer;
			this.socket = socket;
			start();
		}
		public void run()
		{
			try
			{
				//The data coming in
				DataInputStream DataIn = new DataInputStream( socket.getInputStream() );
				//Loop the data in and distribution.
				while (true)
				{
					String message = DataIn.readUTF();
					//distribute the message
					server.sendToAll( message );
					ServerMsg("Srv","Message recieved and being distributed.");
					
				}
			}
			catch(EOFException ie)
			{
				// This doesn't need an error message as we should never get an EndOfFile
			}
			catch(IOException ie)
			{
				// Get some safe method of disconnection
				//ServerMsg("\nError","0100 - Client Dirty Disconnected.\n");
			}
			finally
			{
				// The connection to the client has stopped
				server.removeConnection(socket);
			}
  }
}